"use strict";

exports.__esModule = true;
exports.isRem = isRem;
exports.calcRem = calcRem;
exports.calcUnitNumber = calcUnitNumber;
exports.getRem = getRem;
exports.setRem = setRem;
exports.setDecimalPixelTransformer = setDecimalPixelTransformer;
exports.isUnitNumber = isUnitNumber;
exports.convertUnit = convertUnit;

/**
 * CSS properties which accept numbers but are not in units of "px".
 */
var UNITLESS_NUMBER_PROPS = {
  animationIterationCount: true,
  borderImageOutset: true,
  borderImageSlice: true,
  borderImageWidth: true,
  boxFlex: true,
  boxFlexGroup: true,
  boxOrdinalGroup: true,
  columnCount: true,
  flex: true,
  flexGrow: true,
  flexPositive: true,
  flexShrink: true,
  flexNegative: true,
  flexOrder: true,
  gridRow: true,
  gridColumn: true,
  fontWeight: true,
  // Main-stream browsers(like chrome) will not remove webkit prefix in the short term.
  // ref: CSSOM webkit-based attribute: https://drafts.csswg.org/cssom/#dom-cssstyledeclaration-webkit-cased-attribute
  webkitLineClamp: true,
  lineClamp: true,
  // We make lineHeight default is px that is diff with w3c spec
  // lineHeight: true,
  opacity: true,
  order: true,
  orphans: true,
  tabSize: true,
  widows: true,
  zIndex: true,
  zoom: true,
  // Weex only
  lines: true
};
var REM_REG = /[-+]?\d*\.?\d+(rem|rpx|$)/g;
var GLOBAL_REM_UNIT = '__global_rem_unit__';
var global = typeof window === 'object' ? window : typeof global === 'object' ? global : {};

var decimalPixelTransformer = function decimalPixelTransformer(val) {
  return val;
}; // Default 1 rem to 1 px


if (getRem() === undefined) {
  setRem(1);
}
/**
 * Is string contains rem
 * note: rpx is an alias to rem
 * @param {String} str
 * @returns {Boolean}
 */


function isRem(str) {
  return typeof str === 'string' && REM_REG.test(str);
}
/**
 * Calculate rem to pixels: '1.2rem' => 1.2 * rem
 * @param {String} str
 * @param {Number} rem
 * @returns {String}
 */


function calcRem(str, remUnit) {
  if (remUnit === void 0) {
    remUnit = getRem();
  }

  return str.replace(REM_REG, function (rem) {
    return decimalPixelTransformer(parseFloat(rem) * remUnit) + 'px';
  });
}

function calcUnitNumber(val, remUnit) {
  if (remUnit === void 0) {
    remUnit = getRem();
  }

  return decimalPixelTransformer(val * remUnit) + 'px';
}

function getRem() {
  return global[GLOBAL_REM_UNIT];
}

function setRem(rem) {
  global[GLOBAL_REM_UNIT] = rem;
}
/**
 * Set a function to transform unit of pixel,
 * default to passthrough.
 * @param {Function} transformer function
 */


function setDecimalPixelTransformer(transformer) {
  decimalPixelTransformer = transformer;
}

function isUnitNumber(val, prop) {
  return typeof val === 'number' && !UNITLESS_NUMBER_PROPS[prop];
}

function convertUnit(val, prop, remUnit) {
  if (remUnit === void 0) {
    remUnit = getRem();
  }

  if (prop && isUnitNumber(val, prop)) {
    return calcUnitNumber(val, remUnit);
  } else if (isRem(val)) {
    return calcRem(val, remUnit);
  } else {
    return val;
  }
}